/************************************************************************************
**    SIMphoto is an aplication based on single image photogrammetry. It is useful
**    for creating and printing a photomap. The software is designed for close
**    range photogrammtery. Outputs are usable in the field of heritage preservation.
**    Copyright (C) 2009 David Cizek
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**    This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program. If not, see <http://www.gnu.org/licenses/>.
**
**    How to contact me: cizekdav@gmail.com
**
***********************************************************************************/

#include "marktool.h"

MarkTool::MarkTool(int numberOfPoints)
{
    startMarkingButton = new QPushButton(tr("Start Marking"), this);
    removePointButton = new QPushButton(tr("Remove Last Point"), this);
    openPointTableButton = new QPushButton(tr("Open Point Table"), this);
    exitMarkingButton = new QPushButton(tr("Exit Marking"), this);
    pointNumberSpinBox = new QSpinBox(this);
    pointLabel = new QLabel(this);
    shortcutLabel = new QLabel(tr("'Q' key = forward, 'A' key = back"));

    pointNumberSpinBox->setEnabled(false);
    pointLabel->setEnabled(false);
    removePointButton->setEnabled(false);
    openPointTableButton->setEnabled(false);
    shortcutLabel->setEnabled(false);

    actualPoint = 0;
    pointsCount = 0;
    pointNumberSpinBox->setRange(1, numberOfPoints);

    pointLabel->setPixmap(*setPicture(1));
    pointLabel->adjustSize();

    QVBoxLayout *vLayout = new QVBoxLayout;
    QHBoxLayout *hLayout = new QHBoxLayout;

    hLayout->addWidget(pointNumberSpinBox);
    hLayout->addWidget(pointLabel);

    vLayout->addWidget(startMarkingButton);
    vLayout->addWidget(shortcutLabel);
    vLayout->addLayout(hLayout);
    vLayout->addWidget(removePointButton);
    vLayout->addWidget(openPointTableButton);
    vLayout->addWidget(exitMarkingButton);

    setLayout(vLayout);
    adjustSize();
    setMaximumSize(this->width(), this->height());

    readyToMarking = false;

    connect(pointNumberSpinBox, SIGNAL(valueChanged(int)), this, SLOT(valueWasChanged()));
    connect(startMarkingButton, SIGNAL(clicked()), this, SLOT(startMarking()));
    connect(removePointButton, SIGNAL(clicked()), this, SLOT(removePoint()));
}

//Actual mark is changed
void MarkTool::valueWasChanged()
{
    //If mark doesn't exist in the list of points
    if(pointNumberSpinBox->value() > pointsCount)
    {
        //If a new value is consequent number: mark is added...
        if(pointNumberSpinBox->value() - pointsCount < 2)
        {
            addPoint();
            pointAddedBySpinBox();
        }
        else
        {
            //...else: turn back to current point
            pointNumberSpinBox->setValue(actualPoint);
        }
    }
    else
    {
        //change mark
        pointLabel->setPixmap(*setPicture(pointNumberSpinBox->value()));
        pointLabel->adjustSize();
    }
}

QPixmap* MarkTool::setPicture(int i)
{
    QString pointFile("images/points/");
    pointFile.append(QString::number(i)).append(".bmp");

    QPixmap *pixmap = new QPixmap(pointFile);
    QBitmap pointMask("images/points/mask.bmp");

    pixmap->setMask(pointMask);

    actualPoint = i;

    return pixmap;
    delete pixmap;
}

void MarkTool::startMarking()
{
    startMarkingButton->setEnabled(false);
    pointNumberSpinBox->setEnabled(true);
    pointLabel->setEnabled(true);
    openPointTableButton->setEnabled(true);
    shortcutLabel->setEnabled(true);

    readyToMarking = true;
    readyToMarkingSignal();
}

void MarkTool::addPoint()
{
    pointsCount++;
    actualPoint = pointsCount;

    QLabel *point = new QLabel;
    point->setPixmap(*setPicture(pointsCount));

    pointLabel->setPixmap(*setPicture(pointsCount));

    //appending new point to the list of points
    list.append(point);

    if(pointsCount > 1 && pointNumberSpinBox->maximum() == 9)
        removePointButton->setEnabled(true);
}

void MarkTool::removePoint()
{
    delete list.at(pointsCount - 1);
    list.removeLast();

    pointsCount--;
    actualPoint = pointsCount;

    pointNumberSpinBox->setValue(pointsCount);

    if(pointsCount == 1)
        removePointButton->setEnabled(false);

    //emitting signal:
    pointWasRemoved();
}

QPushButton* MarkTool::getStartButton()
{
    return startMarkingButton;
}

QPushButton* MarkTool::getExitButton()
{
    return exitMarkingButton;
}

bool MarkTool::marking()
{
    return readyToMarking;
}

QLabel* MarkTool::getPoint()
{
    return list.at(pointNumberSpinBox->value() - 1);
}

void MarkTool::closeEvent(QCloseEvent *closeEvent)
{
    for(int i = 0; i < pointsCount; i++)
    {
        delete list.at(i);
    }
}

QSpinBox* MarkTool::getSpinBox()
{
    return pointNumberSpinBox;
}

int MarkTool::getPointsCount()
{
    return pointsCount;
}

QList<QLabel*> MarkTool::getListOfPoints()
{
    return list;
}

QPushButton* MarkTool::getOpenButton()
{
    return openPointTableButton;
}
